# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: serve.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from . import job_pb2 as job__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0bserve.proto\x12\x06\x64\x65ploy\x1a\x1bgoogle/protobuf/empty.proto\x1a google/protobuf/field_mask.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\tjob.proto\"\xed\x01\n\x07Serving\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x1d\n\x06models\x18\x03 \x03(\x0b\x32\r.deploy.Model\x12-\n\x13\x64\x65ployment_endpoint\x18\x04 \x01(\x0b\x32\x10.deploy.Endpoint\x12)\n\x0fmodel_endpoints\x18\x05 \x03(\x0b\x32\x10.deploy.Endpoint\x12 \n\x08resource\x18\x06 \x01(\x0b\x32\x0e.jobs.Resource\x12-\n\x10\x63pu_scale_config\x18\x07 \x01(\x0b\x32\x13.deploy.ScaleConfig\"7\n\x08\x45ndpoint\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0b\n\x03url\x18\x02 \x01(\t\x12\x12\n\naccess_key\x18\x03 \x01(\t\"\xd6\x01\n\rServingStatus\x12\x33\n\rupload_status\x18\x01 \x01(\x0e\x32\x1c.deploy.ServingStatus.Status\x12\x33\n\rdeploy_status\x18\x02 \x01(\x0e\x32\x1c.deploy.ServingStatus.Status\"[\n\x06Status\x12\x16\n\x12STATUS_UNSPECIFIED\x10\x00\x12\x12\n\x0eSTATUS_PENDING\x10\x01\x12\x12\n\x0eSTATUS_SUCCESS\x10\x02\x12\x11\n\rSTATUS_FAILED\x10\x03\"L\n\x0bScaleConfig\x12\x14\n\x0cmin_replicas\x18\x01 \x01(\x05\x12\x14\n\x0cmax_replicas\x18\x02 \x01(\x05\x12\x11\n\tthreshold\x18\x03 \x01(\x02\"X\n\x0eServingRequest\x12$\n\tauth_info\x18\x01 \x01(\x0b\x32\x11.jobs.NBXAuthInfo\x12 \n\x07serving\x18\x02 \x01(\x0b\x32\x0f.deploy.Serving\"\x8f\x01\n\x14UpdateServingRequest\x12$\n\tauth_info\x18\x01 \x01(\x0b\x32\x11.jobs.NBXAuthInfo\x12 \n\x07serving\x18\x02 \x01(\x0b\x32\x0f.deploy.Serving\x12/\n\x0bupdate_mask\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.FieldMask\"Y\n\x12ServingListRequest\x12$\n\tauth_info\x18\x01 \x01(\x0b\x32\x11.jobs.NBXAuthInfo\x12\r\n\x05limit\x18\x02 \x01(\x05\x12\x0e\n\x06offset\x18\x03 \x01(\x05\"K\n\x13ServingListResponse\x12!\n\x08servings\x18\x01 \x03(\x0b\x32\x0f.deploy.Serving\x12\x11\n\tnext_page\x18\x02 \x01(\x05\"\xec\x03\n\x05Model\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x18\n\x10serving_group_id\x18\x03 \x01(\t\x12.\n\ncreated_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\x04type\x18\x05 \x01(\x0e\x32\x19.deploy.Model.ServingType\x12 \n\x08resource\x18\x06 \x01(\x0b\x32\x0e.jobs.Resource\x12\x1c\n\x04\x63ode\x18\x07 \x01(\x0b\x32\x0e.jobs.Job.Code\x12\"\n\x08\x65ndpoint\x18\x08 \x01(\x0b\x32\x10.deploy.Endpoint\x12+\n\npin_status\x18\t \x01(\x0e\x32\x17.deploy.Model.PinStatus\x12%\n\x06status\x18\x0e \x01(\x0b\x32\x15.deploy.ServingStatus\"E\n\x0bServingType\x12\x1c\n\x18SERVING_TYPE_UNSPECIFIED\x10\x00\x12\x18\n\x14SERVING_TYPE_NBOX_OP\x10\x01\"W\n\tPinStatus\x12\x1a\n\x16PIN_STATUS_UNSPECIFIED\x10\x00\x12\x15\n\x11PIN_STATUS_PINNED\x10\x01\x12\x17\n\x13PIN_STATUS_UNPINNED\x10\x02\"R\n\x0cModelRequest\x12$\n\tauth_info\x18\x01 \x01(\x0b\x32\x11.jobs.NBXAuthInfo\x12\x1c\n\x05model\x18\x02 \x01(\x0b\x32\r.deploy.Model\"k\n\x10ModelListRequest\x12$\n\tauth_info\x18\x01 \x01(\x0b\x32\x11.jobs.NBXAuthInfo\x12\x12\n\nserving_id\x18\x02 \x01(\t\x12\r\n\x05limit\x18\x03 \x01(\x05\x12\x0e\n\x06offset\x18\x04 \x01(\x05\"E\n\x11ModelListResponse\x12\x1d\n\x06models\x18\x01 \x03(\x0b\x32\r.deploy.Model\x12\x11\n\tnext_page\x18\x02 \x01(\x05\"$\n\x08ModelLog\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04logs\x18\x02 \x03(\t2\xc4\x02\n\x0eServingService\x12\x31\n\x06\x43reate\x12\x16.deploy.ServingRequest\x1a\x0f.deploy.Serving\x12\x35\n\nGetServing\x12\x16.deploy.ServingRequest\x1a\x0f.deploy.Serving\x12G\n\x0cListServings\x12\x1a.deploy.ServingListRequest\x1a\x1b.deploy.ServingListResponse\x12?\n\rDeleteServing\x12\x16.deploy.ServingRequest\x1a\x16.google.protobuf.Empty\x12>\n\rUpdateServing\x12\x1c.deploy.UpdateServingRequest\x1a\x0f.deploy.Serving2\x9d\x03\n\x0cModelService\x12\x36\n\x05Watch\x12\x14.deploy.ModelRequest\x1a\x15.deploy.ServingStatus0\x01\x12\x32\n\x0bUploadModel\x12\x14.deploy.ModelRequest\x1a\r.deploy.Model\x12-\n\x06\x44\x65ploy\x12\x14.deploy.ModelRequest\x1a\r.deploy.Model\x12\x35\n\tModelLogs\x12\x14.deploy.ModelRequest\x1a\x10.deploy.ModelLog0\x01\x12\x41\n\nListModels\x12\x18.deploy.ModelListRequest\x1a\x19.deploy.ModelListResponse\x12;\n\x0b\x44\x65leteModel\x12\x14.deploy.ModelRequest\x1a\x16.google.protobuf.Empty\x12;\n\x0bSetModelPin\x12\x14.deploy.ModelRequest\x1a\x16.google.protobuf.EmptyB4Z2github.com/NimbleBoxAI/jobs-architecture/deploy_pbb\x06proto3')



_SERVING = DESCRIPTOR.message_types_by_name['Serving']
_ENDPOINT = DESCRIPTOR.message_types_by_name['Endpoint']
_SERVINGSTATUS = DESCRIPTOR.message_types_by_name['ServingStatus']
_SCALECONFIG = DESCRIPTOR.message_types_by_name['ScaleConfig']
_SERVINGREQUEST = DESCRIPTOR.message_types_by_name['ServingRequest']
_UPDATESERVINGREQUEST = DESCRIPTOR.message_types_by_name['UpdateServingRequest']
_SERVINGLISTREQUEST = DESCRIPTOR.message_types_by_name['ServingListRequest']
_SERVINGLISTRESPONSE = DESCRIPTOR.message_types_by_name['ServingListResponse']
_MODEL = DESCRIPTOR.message_types_by_name['Model']
_MODELREQUEST = DESCRIPTOR.message_types_by_name['ModelRequest']
_MODELLISTREQUEST = DESCRIPTOR.message_types_by_name['ModelListRequest']
_MODELLISTRESPONSE = DESCRIPTOR.message_types_by_name['ModelListResponse']
_MODELLOG = DESCRIPTOR.message_types_by_name['ModelLog']
_SERVINGSTATUS_STATUS = _SERVINGSTATUS.enum_types_by_name['Status']
_MODEL_SERVINGTYPE = _MODEL.enum_types_by_name['ServingType']
_MODEL_PINSTATUS = _MODEL.enum_types_by_name['PinStatus']
Serving = _reflection.GeneratedProtocolMessageType('Serving', (_message.Message,), {
  'DESCRIPTOR' : _SERVING,
  '__module__' : 'serve_pb2'
  # @@protoc_insertion_point(class_scope:deploy.Serving)
  })
_sym_db.RegisterMessage(Serving)

Endpoint = _reflection.GeneratedProtocolMessageType('Endpoint', (_message.Message,), {
  'DESCRIPTOR' : _ENDPOINT,
  '__module__' : 'serve_pb2'
  # @@protoc_insertion_point(class_scope:deploy.Endpoint)
  })
_sym_db.RegisterMessage(Endpoint)

ServingStatus = _reflection.GeneratedProtocolMessageType('ServingStatus', (_message.Message,), {
  'DESCRIPTOR' : _SERVINGSTATUS,
  '__module__' : 'serve_pb2'
  # @@protoc_insertion_point(class_scope:deploy.ServingStatus)
  })
_sym_db.RegisterMessage(ServingStatus)

ScaleConfig = _reflection.GeneratedProtocolMessageType('ScaleConfig', (_message.Message,), {
  'DESCRIPTOR' : _SCALECONFIG,
  '__module__' : 'serve_pb2'
  # @@protoc_insertion_point(class_scope:deploy.ScaleConfig)
  })
_sym_db.RegisterMessage(ScaleConfig)

ServingRequest = _reflection.GeneratedProtocolMessageType('ServingRequest', (_message.Message,), {
  'DESCRIPTOR' : _SERVINGREQUEST,
  '__module__' : 'serve_pb2'
  # @@protoc_insertion_point(class_scope:deploy.ServingRequest)
  })
_sym_db.RegisterMessage(ServingRequest)

UpdateServingRequest = _reflection.GeneratedProtocolMessageType('UpdateServingRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATESERVINGREQUEST,
  '__module__' : 'serve_pb2'
  # @@protoc_insertion_point(class_scope:deploy.UpdateServingRequest)
  })
_sym_db.RegisterMessage(UpdateServingRequest)

ServingListRequest = _reflection.GeneratedProtocolMessageType('ServingListRequest', (_message.Message,), {
  'DESCRIPTOR' : _SERVINGLISTREQUEST,
  '__module__' : 'serve_pb2'
  # @@protoc_insertion_point(class_scope:deploy.ServingListRequest)
  })
_sym_db.RegisterMessage(ServingListRequest)

ServingListResponse = _reflection.GeneratedProtocolMessageType('ServingListResponse', (_message.Message,), {
  'DESCRIPTOR' : _SERVINGLISTRESPONSE,
  '__module__' : 'serve_pb2'
  # @@protoc_insertion_point(class_scope:deploy.ServingListResponse)
  })
_sym_db.RegisterMessage(ServingListResponse)

Model = _reflection.GeneratedProtocolMessageType('Model', (_message.Message,), {
  'DESCRIPTOR' : _MODEL,
  '__module__' : 'serve_pb2'
  # @@protoc_insertion_point(class_scope:deploy.Model)
  })
_sym_db.RegisterMessage(Model)

ModelRequest = _reflection.GeneratedProtocolMessageType('ModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _MODELREQUEST,
  '__module__' : 'serve_pb2'
  # @@protoc_insertion_point(class_scope:deploy.ModelRequest)
  })
_sym_db.RegisterMessage(ModelRequest)

ModelListRequest = _reflection.GeneratedProtocolMessageType('ModelListRequest', (_message.Message,), {
  'DESCRIPTOR' : _MODELLISTREQUEST,
  '__module__' : 'serve_pb2'
  # @@protoc_insertion_point(class_scope:deploy.ModelListRequest)
  })
_sym_db.RegisterMessage(ModelListRequest)

ModelListResponse = _reflection.GeneratedProtocolMessageType('ModelListResponse', (_message.Message,), {
  'DESCRIPTOR' : _MODELLISTRESPONSE,
  '__module__' : 'serve_pb2'
  # @@protoc_insertion_point(class_scope:deploy.ModelListResponse)
  })
_sym_db.RegisterMessage(ModelListResponse)

ModelLog = _reflection.GeneratedProtocolMessageType('ModelLog', (_message.Message,), {
  'DESCRIPTOR' : _MODELLOG,
  '__module__' : 'serve_pb2'
  # @@protoc_insertion_point(class_scope:deploy.ModelLog)
  })
_sym_db.RegisterMessage(ModelLog)

_SERVINGSERVICE = DESCRIPTOR.services_by_name['ServingService']
_MODELSERVICE = DESCRIPTOR.services_by_name['ModelService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2github.com/NimbleBoxAI/jobs-architecture/deploy_pb'
  _SERVING._serialized_start=131
  _SERVING._serialized_end=368
  _ENDPOINT._serialized_start=370
  _ENDPOINT._serialized_end=425
  _SERVINGSTATUS._serialized_start=428
  _SERVINGSTATUS._serialized_end=642
  _SERVINGSTATUS_STATUS._serialized_start=551
  _SERVINGSTATUS_STATUS._serialized_end=642
  _SCALECONFIG._serialized_start=644
  _SCALECONFIG._serialized_end=720
  _SERVINGREQUEST._serialized_start=722
  _SERVINGREQUEST._serialized_end=810
  _UPDATESERVINGREQUEST._serialized_start=813
  _UPDATESERVINGREQUEST._serialized_end=956
  _SERVINGLISTREQUEST._serialized_start=958
  _SERVINGLISTREQUEST._serialized_end=1047
  _SERVINGLISTRESPONSE._serialized_start=1049
  _SERVINGLISTRESPONSE._serialized_end=1124
  _MODEL._serialized_start=1127
  _MODEL._serialized_end=1619
  _MODEL_SERVINGTYPE._serialized_start=1461
  _MODEL_SERVINGTYPE._serialized_end=1530
  _MODEL_PINSTATUS._serialized_start=1532
  _MODEL_PINSTATUS._serialized_end=1619
  _MODELREQUEST._serialized_start=1621
  _MODELREQUEST._serialized_end=1703
  _MODELLISTREQUEST._serialized_start=1705
  _MODELLISTREQUEST._serialized_end=1812
  _MODELLISTRESPONSE._serialized_start=1814
  _MODELLISTRESPONSE._serialized_end=1883
  _MODELLOG._serialized_start=1885
  _MODELLOG._serialized_end=1921
  _SERVINGSERVICE._serialized_start=1924
  _SERVINGSERVICE._serialized_end=2248
  _MODELSERVICE._serialized_start=2251
  _MODELSERVICE._serialized_end=2664
# @@protoc_insertion_point(module_scope)
